--[[
MultiTool Multi Boxing utilities inspired by smurfys MultiBoxer addon

Shouts to http://old.wowace.com/wiki/WelcomeHome_-_Your_first_Ace3_Addon

Extra special shouts to Allara and OrionShock for pointing me in the right direction RE: AceComm-3.0

Author: DigitalSorceress
Date: 2008/11/11
Version: 0.1.01
]]-

local _myName = false
local _myVersion = false

MultiTool = LibStub("AceAddon-3.0"):NewAddon("MultiTool", "AceConsole-3.0","AceEvent-3.0","AceSerializer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MultiTool")

-- ------------------------------------------------
-- CONFIGURATION STUFF
-- ------------------------------------------------
-- Holds our configuration options
local options = {
  name = "MultiTool",
  handler = MultiTool,
  type = 'group',
  args = {
    enable={
      name=L["ENABLE_NAME"],
      desc=L["ENABLE_DESC"],
      type="toggle",
      get = function(info) return MultiTool.enabled end
      set = function(info,val) MultiTool.enabled = val end
    },
   debugFlag = {
      type = "toggle",
      name = L["DEBUGGING_NAME"],
      desc = L["DEBUGGING_DESC"],
      get = "getDebugFlag",
      set = "setDebugFlag"
    },
   duelFlag = {
      type = "toggle",
      name = L["DUELING_NAME"],
      desc = L["DUELING_DESC"],
      get = "getDuelFlag",
      set = "setDuelFlag"
    },
    myMsg = {
      type = "input",
      name = L["MESSAGE_NAME"],
      desc = L["MESSAGE_DESC"],
      usage = L["MESSAGE_USAGE"],
      get = "getMyMsg",
      set = "setMyMsg"
    },
    showInChat = {
      type = "toggle",
      name = L["SHOW_IN_CHAT"],
      desc = L["SHOW_IN_CHAT_DESC"],
      get = "getShowInChat",
      set = "setShowInChat"
    },
    showOnScreen = {
      type = "toggle",
      name = L["SHOW_ON_SCREEN"],
      desc = L["SHOW_ON_SCREEN_DESC"],
      get = "getShowOnScreen",
      set = "setShowOnScreen"
    }
  }
}

local defaults = {
  profile = {
    myMsg = L["MESSAGE_DEFAULT"],
    showInChat = false,
    showOnScreen = true,
    debugFlag = true,
    duelFlag = true
  }
}


--
-- Field handlers
--
function MultiTool:getMyMsg(info)
  return self.db.profile.myMsg
end

function MultiTool:setMyMsg(info, newValue)
  self.db.profile.myMsg = newValue
end

function MultiTool:getShowInChat(info)
    return self.db.profile.showInChat
end

function MultiTool:setShowInChat(info, value)
    self.db.profile.showInChat = value
end

function MultiTool:getDebugFlag(info)
  return self.db.profile.debugFlag
end
  
function MultiTool:setDebugFlag(info, value)
  self.db.profile.debugFlag = value
end

function MultiTool:getShowOnScreen(info)
    return self.db.profile.showOnScreen
end

function MultiTool:setShowOnScreen(info, value)
    self.db.profile.showOnScreen = value
end

function MultiTool:getDuelFlag(info)
  return self.db.profile.duelFlag
end
  
function MultiTool:setDuelFlag(info, value)
  self.db.profile.duelFlag = value
end


--
-- Chat Command setup
-- 
function MultiTool:ChatCommand(input)
  if not input or input:trim() == "" then
    -- This would use the Ace config gui
    -- LibStub("AceConfigDialog-3.0"):Open("MultiTool")
    -- This uses the Bliz options pane instead
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(MultiTool, L["SHORT_SLASH_CMD"], L["FULL_SLASH_CMD"], input)
  end
end



--MultiTool.OnCommReceive = {}

-- ------------------------------------------------
-- BASIC REQUIRED FUNCTIONS
-- ------------------------------------------------

-- Called when the addon is loaded
function MultiTool:OnInitialize()
  -- Using the AceDB for storage of our info
  self.db = LibStub("AceDB-3.0"):New("MultiToolDB", defaults, "Default")

  -- main options table stuff
  LibStub("AceConfig-3.0"):RegisterOptionsTable("MultiTool", options)
  -- adding to Blizzard options configuration
  self.optionsFrame = LibStub("AceConfig-3.0"):AddToBlizOptions("MultiTool", "MultiTool")
  -- registering the slash commands
  self:RegisterChatCommand(L["FULL_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand(L["SHORT_SLASH_CMD"], "ChatCommand")
  
  RegisterComm("MultiTool","OnCommReceived")
  
  -- Stole this from smurphys MultiBoxer
	self:AddEventListener("DUEL_REQUESTED")
  
  -- Some initialization of our happy local vars
  MultiTool._myName = "MultiTool"
  MultiTool._myVersion = "0.1"
end


-- Called when the addon is enabled
function MultiTool:OnEnable()
  -- just a test register event
  self:RegisterEvent("ZONE_CHANGED", zoneChanged)
  self:RegisterEvent("DUEL_REQUESTED", duelRequested, ...)
  
	hooksecurefunc("TakeTaxiNode",TakeTaxiNodeHook)

  if (self.db.profile.debugFlag) then
    self:Print(_myName.." v".._myVersion.." ENABLED!")
  end
end


-- Called when the addon is disabled
function MultiTool:OnDisable()
  --
end


-- ------------------------------------------------
-- EVENT HANDLERS
-- ------------------------------------------------

function MultiTool:zoneChanged()
  if GetBindLocation() == GetSubZoneText() then
    -- CHAT
    if (self.db.profile.showInChat) then
      self:Print(self.db.profile.message)
    end
    
    -- IN-YER-FACE
    if (self.db.profile.showOnScreen) then
      UIErrorsFrame:AddMessage(self.db.profile.message, 1.0, 1.0, 1.0, 5.0)
    end
  end
end


--
-- If profile is set for denying duels, then autocancels all incoming duels
--
function MultiTool:duelRequested(challenger)
	if (self.db.profile.duelFlag) then
    if (self.debugFlag) then
      self:Print("Duel denied from challenger:"..challenger)
    end
    CancelDuel();
  	StaticPopup_Hide("DUEL_REQUESTED")
	end
end

-- ------------------------------------------------
-- HOOKED FUNCTIONS
-- ------------------------------------------------
function TakeTaxiNodeHook(index) 
  local prefix = "MultiTool"
  
  local msg = {
    op = "TAKETAXI",
    node = index
  }
  local text = self:Serialize(msg)
  
  local distribution = "RAID"
  local sender = player
  local prio = "ALERT"
  
  --debug
	self:Print("SentBroadcast "..msg["index"].." : "..TaxiNodeName(index))
  self:SendCommMessage(prefix, msg, distribution, sender, prio)
end

-- ------------------------------------------------
-- COMM-BASED FUNCTIONS
-- ------------------------------------------------

function MultiTool:OnCommReceived(prefix, message, distribution, sender, priority)
  local success, o = self:Deserialize(message)
  if (success == false) then
    return -- Failure
  else
    -- Do something with the data
    -- Digitalsorceress :: Added my proposed functionality
    if (o["op"] == "TAXI_DISPATCH" and sender ~= UnitName("player")) then
      TakeTaxiNode(o["node"])
    end
  end
end

